//----------------------------------------------------------------------------
// TSplitWindow - (C) Copyright 1995 by Kent Reisdorph
//
// Source for class TSplitWindow, a class allow horizontal
// or vertical splitting and resizing of two windows.
//
//----------------------------------------------------------------------------
#include "splitwin.h"

DEFINE_RESPONSE_TABLE1(TSplitWindow, TLayoutWindow)
	EV_WM_LBUTTONDOWN,
	EV_WM_LBUTTONUP,
	EV_WM_MOUSEMOVE,
	EV_WM_SETCURSOR,
END_RESPONSE_TABLE;

// Constructor. Sets up child windows. Sets Layout.
TSplitWindow::TSplitWindow(TWindow* parent, const char far* title,
		TWindow* child1, TWindow* child2, int child1Size,
		BOOL vertical, TModule* module)
	: TLayoutWindow(parent, title, module)
{
	childLeft = child1;
	childLeft->SetParent(this);
	childLeft->EnableAutoCreate();
	childRight = child2;
	childRight->SetParent(this);
	childRight->EnableAutoCreate();
	leftChildSize = child1Size;
	isVertical = vertical;
	if (isVertical) LayoutVertical();
	else LayoutHorizontal();
	dragging = FALSE;
	dc = 0;
}

TSplitWindow::~TSplitWindow()
{
	if (dc) delete dc;
}

// EvLButtonDown starts the dragging process. Create a dc for
// our use while we are dragging the sizing bar. Sets ROP code
// to R2_NOT so that the window underneath the bar is visible
// through the bar. This mode also makes erasing the bar as
// simple as redrawing it at the same location.
void
TSplitWindow::EvLButtonDown(UINT, TPoint& point)
{
	offScreen = FALSE;
	SetCapture();
	dragging = TRUE;
	dc = new TWindowDC(*this);
	dc->SetROP2(R2_NOT);
	TRect rect = GetClientRect();
	// Need client area width and height so we know how far to draw
	// the sizing bar.
	clientHeight = rect.Height();
	clientWidth = rect.Width();
	// Select a black pen the width of the sizing border.
	dc->SelectObject(TPen(TColor::Black, GetSystemMetrics(SM_CXFRAME)));
	// Draw vertical bar if isVertical == TRUE otherwise draw a
	// horizontal bar.
	if (isVertical) {
		dc->MoveTo(point.x, 0);
		dc->LineTo(point.x, clientHeight);
	}
	else {
		dc->MoveTo(0, point.y);
		dc->LineTo(clientWidth, point.y);
	}
	// Need to save this TPoint so we can use it to erase the
	// line later.
	oldPoint = point;
}

// EvLButtonUp. Stops the dragging process. Resets the layout of
// the windows. Erases the last line drawn.
void
TSplitWindow::EvLButtonUp(UINT, TPoint& point)
{
	dragging = FALSE;
	if (isVertical) {
		dc->MoveTo(oldPoint.x, 0);
		dc->LineTo(oldPoint.x, clientHeight);
	}
	else {
		dc->MoveTo(0, oldPoint.y);
		dc->LineTo(clientWidth, oldPoint.y);
	}
	ReleaseCapture();
	SetNewPositions(point);
	delete dc;
}

// EvMouseMove. Erases and redraws the line. Sets the cursor
// back to the arrow if it moves off our window. Resets the
// cursor to the sizing cursor when the cursor moves back over
// our window. Calls DrawSizingLine for actual line drawing.
void
TSplitWindow::EvMouseMove(UINT, TPoint& point)
{
	if (!dragging) return;
	if (point.x < 0 || point.x > clientWidth ||
			point.y < 0 || point.x > clientWidth) {
		if (!offScreen) ::SetCursor(LoadCursor(NULL, IDC_ARROW));
		offScreen = TRUE;
	}
	else if (offScreen) {
		offScreen = FALSE;
		::SetCursor(LoadCursor(NULL, arrowType));
	}
	if (!offScreen) DrawSizingBar(point);
}

// Sets the cursor to the sizing cursor if it is over our
// window as opposed to one of the child windows. Resets the
// cursor back to the arrow when we're done.
BOOL
TSplitWindow::EvSetCursor(HWND hwnd, UINT, UINT msg)
{
	if (msg == WM_MOUSEMOVE) {
		if (hwnd == *this) ::SetCursor(LoadCursor(NULL, arrowType));
		else  if (!dragging) ::SetCursor(LoadCursor(NULL, IDC_ARROW));
	}
	return FALSE;
}

// Layout for Vertical split window. Default setup is 50% for
// each window. The gap is determined by the size of a sizeable
// frame - (2 * the size of a non-sizeable border). The size
// SM_CXFRAME was too large. The top left corner of the left
// child is anchored at 0,0. The bottom is the bottom of the parent.
// The width is set to 50% of the parent. The right child is
// the width of the gap to the right of the left child. The top
// and bottom edges are set to the size of the parent. The right
// edge is set to the right edge of the parent.
// SetChildLayoutMetrics sets the metrics for the child and
// TLayoutWindow::Layout() sets the layout once the layout metrics
// are set for each child.
void
TSplitWindow::LayoutVertical()
{
	TLayoutMetrics lm;
	childLeft->Attr.Style |= WS_CLIPSIBLINGS | WS_BORDER;
	lm.X.Absolute(lmLeft, 0);
	lm.Y.Absolute(lmTop, 0);
	lm.Width.PercentOf(lmParent, leftChildSize);
	lm.Height.SameAs(lmParent, lmHeight);
	SetChildLayoutMetrics(*childLeft, lm);
	childRight->Attr.Style |= WS_CLIPSIBLINGS | WS_BORDER;
	int gap = GetSystemMetrics(SM_CXFRAME) -
		(GetSystemMetrics(SM_CYBORDER) *2);
	lm.X.RightOf(childLeft, gap);
	lm.Y.Absolute(lmTop, 0);
	lm.Width.SameAs(lmParent, lmRight);
	lm.Height.SameAs(lmParent, lmHeight);
	SetChildLayoutMetrics(*childRight, lm);
	Layout();
	arrowType = (char*)IDC_SIZEWE;
}

// Same as above only different.
void
TSplitWindow::LayoutHorizontal()
{
	TLayoutMetrics lm;
	childLeft->Attr.Style |= WS_CLIPSIBLINGS | WS_BORDER;
	lm.X.Absolute(lmLeft, 0);
	lm.Y.Absolute(lmTop, 0);
	lm.Width.SameAs(lmParent, lmWidth);
	lm.Height.PercentOf(lmParent, leftChildSize);
	SetChildLayoutMetrics(*childLeft, lm);
	childRight->Attr.Style |= WS_CLIPSIBLINGS | WS_BORDER;
	int gap = GetSystemMetrics(SM_CXFRAME) -
		(GetSystemMetrics(SM_CYBORDER) *2);
	lm.X.Absolute(lmLeft, 0);
	lm.Y.Below(childLeft, gap);
	lm.Width.SameAs(lmParent);
	lm.Height.SameAs(lmParent, lmBottom);
	SetChildLayoutMetrics(*childRight, lm);
	Layout();
	arrowType = (char*)IDC_SIZENS;
}

// Switches from a vertical split window to a horizontal split
// window and vice versa.
void
TSplitWindow::Switch()
{
	if (isVertical) {
		isVertical = FALSE;
		LayoutHorizontal();
	}
	else {
		isVertical = TRUE;
		LayoutVertical();
	}
}

// SetNewPositions resets the layout metric for each child and
// calls Layout() to realize the changes. The layout is set as
// a percentage of the parent. Future implementations might want
// the layout to be an absolute location rather than a percentage.
void
TSplitWindow::SetNewPositions(TPoint& point)
{
	TRect rect = GetClientRect();
	TLayoutMetrics lm;
	lm.X.Absolute(lmLeft, 0);
	lm.Y.Absolute(lmTop, 0);
	if (isVertical) {
		if (point.x < 10) point.x = 10;
		lm.Width.PercentOf(lmParent, (point.x*100L)/rect.Width());
		lm.Height.SameAs(lmParent, lmHeight);
	}
	else {
		if (point.y < 10) point.y = 10;
		lm.Width.SameAs(lmParent, lmWidth);
		lm.Height.PercentOf(lmParent, (point.y*100L)/rect.Height());
	}
	SetChildLayoutMetrics(*childLeft, lm);
	Layout();
}

// DrawSizingBar. First erases the old line by drawing a line at
// the old location. Then we draw a line at the new location and
// save the new location in oldPoint so we can erase later.
void
TSplitWindow::DrawSizingBar(TPoint& point)
{
	if (isVertical) {
		dc->MoveTo(oldPoint.x, 0);
		dc->LineTo(oldPoint.x, clientHeight);
		dc->MoveTo(point.x, 0);
		dc->LineTo(point.x, clientHeight);
	}
	else {
		dc->MoveTo(0, oldPoint.y);
		dc->LineTo(clientWidth, oldPoint.y);
		dc->MoveTo(0, point.y);
		dc->LineTo(clientWidth, point.y);
	}
	oldPoint = point;
}

// SetLeftChild. Deletes the old client and creates a new one
// returns a pointer to the new child. Returns 0 if we were passed
// a 0 as the new window. Without this the app will crash on the
// SetParent() call. Layout the windows as we do in the constructor.
TWindow*
TSplitWindow::SetLeftChild(TWindow* client)
{
	if (!client) return 0;
	if (childLeft) delete childLeft;
	childLeft = client;
	childLeft->SetParent(this);
	childLeft->EnableAutoCreate();
	childLeft->Create();
	if (childLeft) {
		if (isVertical) LayoutVertical();
		else LayoutHorizontal();
	}
	return childLeft;
}

// SetRightClient. Same as SetLeftChild above.
TWindow*
TSplitWindow::SetRightChild(TWindow* client)
{
	if (!client) return 0;
	if (childRight) delete childRight;
	childRight = client;
	childRight->SetParent(this);
	childRight->EnableAutoCreate();
	childRight->Create();
	if (childRight) {
		if (isVertical) LayoutVertical();
		else LayoutHorizontal();
	}
	return childRight;
}

